@@ -268,9 +268,9 @@ class Cursor(DBAPICursor):
         """Clears the tmp data"""
         self.connection.clear(self._query_id)
 
-    def cancel(self):
+    def cancel(self, query_id):
         _logger.info("Cancelling query")
-        self.connection.query_cancel(self._query_id)
+        self.connection.query_cancel(query_id)
 
     def execute(self, operation, parameters=None, **kwargs):
         """Prepare and execute a database operation (query or command).
@@ -293,7 +293,7 @@ class Cursor(DBAPICursor):
         buffer = BytesIO(buffer)
         self._rowcount, self._query_columns_description = get_query_columns_info(buffer)
         _logger.info("finished populating metadata")
-        return
+        return self._query_id
 
     def rowcount(self):
         _logger.info("fetch returning the rowcount from e6x itself")
@@ -322,7 +322,9 @@ class Cursor(DBAPICursor):
             if rows is None:
                 break
             self._data = self._data + rows
-        return self._data
+        rows = self._data
+        self._data = None
+        return rows
 
     def fetch_batch(self):
         _logger.debug("fetching next batch from e6data")
@@ -350,7 +352,7 @@ class Cursor(DBAPICursor):
         while len(self._data) < size:
             _logger.info("fetching next batch from fetch many")
             rows = self.fetch_batch()
-            if rows is None or len(rows) == 0:
+            if rows is None:
                 break
             self._data += rows
         _logger.info(len(self._data))
